#include <stdlib.h>

#include "ndalloc.h"

static void *
ndalloc_dims (size_t ndims, size_t size, size_t dims[]) {
  size_t i;
  size_t nelems;
  size_t offset;

  void ** ret;

  if (ndims == 1)
    return malloc(dims[0] * size);

  nelems = dims[0];
  offset = dims[1];

  ret = malloc(nelems * sizeof(void *));

  if (ret == NULL)
    goto error;

  dims[1] *= nelems;
  ret[0] = ndalloc_dims(ndims-1, size, dims+1);

  if (*ret == NULL)
    goto error;

  offset *= ndims > 2 ? sizeof(void *) : size;
  for (i = 1; i < nelems; i++)
    ret[i] = ((char *) ret[0]) + i*offset;

  return ret;

 error:
  free(ret);

  return NULL;
}

void *
ndalloc (size_t ndims, size_t size, ...) {
  size_t i;
  size_t * dims;
  void * ptr;
  va_list ap;

  if (ndims == 0)
    return NULL;

  dims = malloc(ndims * sizeof(size_t));

  if (dims == NULL)
    return NULL;

  va_start(ap, size);
  for (i = 0; i < ndims; i++)
    dims[i] = va_arg(ap, size_t);
  va_end(ap);

  ptr = ndalloc_dims(ndims, size, dims);

  free(dims);
  return ptr;
}

void
ndalloc_free (size_t ndims, void * ptr) {
  if (ndims == 0 || ptr == NULL)
    return;

  if (ndims > 1)
    ndalloc_free(ndims-1, * (void **) ptr);

  free(ptr);
}
