#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/time.h>

#define N 10000000

#define ERROR(msg)				\
  do {						\
    perror(msg);				\
    exit(EXIT_FAILURE);				\
  } while (0)

static double wtime (void) {
  struct timeval now;

  (void) gettimeofday(&now, NULL);

  return now.tv_sec + 1e-6*now.tv_usec;
}

static void write_files (void) {
  size_t i;
  double * values;
  double timer;
  double dx = 1.0/N;

  FILE * ascii;
  FILE * binary;

  values = malloc(N * sizeof(double));

  if (values == NULL)
    ERROR("malloc");

  ascii  = fopen("values.asc", "w");
  binary = fopen("values.bin", "w");

  if (ascii == NULL || binary == NULL)
    ERROR("fopen");

  for (i = 0; i < N; i++)
    values[i] = sin(2*M_PI*i*dx);

  timer = wtime();
  for (i = 0; i < N; i++)
    fprintf(ascii, "%.15f\n", values[i]);
  fflush(ascii);
  timer = wtime() - timer;

  printf("ascii write: %.5f seconds\n", timer);

  timer = wtime();
  fwrite(values, sizeof(double), N, binary);
  fflush(binary);
  timer = wtime() - timer;

  printf("binary write: %.5f seconds\n", timer);

  if (fclose(ascii) == EOF || fclose(binary) == EOF)
    ERROR("fclose");

  free(values);
}

static void read_files (void) {
  size_t i;
  double * values;
  double timer;

#define BUFFER_SIZE 256
  char input_buffer[BUFFER_SIZE];

  FILE * ascii;
  FILE * binary;

  values = malloc(N * sizeof(double));

  if (values == NULL)
    ERROR("malloc");

  ascii  = fopen("values.asc", "r");
  binary = fopen("values.bin", "r");

  if (ascii == NULL || binary == NULL)
    ERROR("fopen");

  timer = wtime();
  i = 0;
  while (fgets(input_buffer, BUFFER_SIZE, ascii) != NULL)
    values[i++] = strtod(input_buffer, NULL);
  timer = wtime() - timer;

  printf("ascii read: %.5f seconds\n", timer);

  timer = wtime();
  fread(values, sizeof(double), N, binary);
  timer = wtime() - timer;

  printf("binary read: %.5f seconds\n", timer);

  if (fclose(ascii) == EOF || fclose(binary) == EOF)
    ERROR("fclose");

  free(values);
}

int main (void) {
  write_files();
  read_files();

  return 0;
}
